"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Request = void 0;
var https_1 = require("https");
var makeFormDate_1 = require("./helpers/makeFormDate");
var parseResponse_1 = require("./helpers/parseResponse");
var parseUrl_1 = require("./helpers/parseUrl");
var Request = /** @class */ (function () {
    // TODO: support before/after filters
    function Request() {
    }
    Request.prototype.request = function (url, method, options) {
        options = __assign({}, options);
        var isForm = options.form === true;
        var hasData = (method === 'POST' || method === 'PUT') && options.data;
        var _a = options.headers, headers = _a === void 0 ? {} : _a;
        var data;
        var form;
        if (hasData) {
            if (isForm) {
                form = makeFormDate_1.makeFormDate(options.data);
                headers = __assign(__assign({}, headers), form.getHeaders());
            }
            else {
                data = JSON.stringify(options.data);
                headers['Content-Type'] = 'application/json';
            }
        }
        // NOTE: is header 'Content-Length': data.length needed?
        var requestOptions = __assign(__assign({}, parseUrl_1.parseUrl(url, options.params)), { port: options.port, method: method,
            headers: headers });
        return new Promise(function (resolve, reject) {
            var req = https_1.request(requestOptions, function (res) {
                var status = res.statusCode;
                var body = '';
                res.on('data', function (chunk) {
                    body += chunk;
                });
                res.on('end', function () {
                    var result = parseResponse_1.parseResponse(status, body);
                    if (status >= 200 && status < 300) {
                        resolve(result);
                    }
                    else {
                        reject(result);
                    }
                });
            });
            req.on('error', function (error) {
                reject({
                    status: 500,
                    data: {
                        code: 'https_system_error',
                        error: error,
                    },
                });
            });
            if (hasData) {
                if (isForm) {
                    form.pipe(req);
                }
                else {
                    req.write(data);
                }
            }
            req.end();
        });
    };
    return Request;
}());
exports.Request = Request;
